import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.util.*;
import java.io.*;
import menu.*;
import store.*;
/**
* An AccountPage is used to edit a UserAccount object.  The
* user edits the fields of the object and then the inputs
* are stored in the associated user data object.
*/
public class AccountPage extends Page {
	/**
	* Initialize a new AccountPage object using the supplied parameters.
	* @param label short label for this page (as might be used in
	* a menu)
	* @param title longer label for this page (as might be used in
	* a screen title)
	* @param icon small icon for this page (as might be used in
	* a toolbar or menu
	* @param obj an instance of StorableObject.  The 
	* StorableObject contains a user data reference that 
	* points to an instance of a UserAccount.
	*/
	public AccountPage(String label,String title,Image icon,StorableObject obj) {
		super(label,title,icon);
		if (obj==null) 
			throw new ClassCastException("StorableObject must be supplied to "+this.getClass().getName());
		if ((obj.getUserData()==null) || (!(obj.getUserData() instanceof UserAccount)))
			throw new ClassCastException("StorableObject must include instanceof UserAccount in "+this.getClass().getName());
		this.sObj = obj;
	}
	/**
	* Initialize a new AccountPage object using the supplied parameters
	* and default values where needed.  The title defaults to null
	* (meaning that the label will be used in its place).  The icon
	* defaults to null (meaning text only will be displayed).
	* @param label short label for this page (as might be used in
	* a menu)
	* @param obj an instance of StorableObject.  The 
	* StorableObject contains a user data reference that 
	* points to an instance of a UserAccount.
	*/
	public AccountPage(String label,StorableObject obj) {
		this(label,null,null,obj);
	}
	/**
	* Process the user command and return an integer value telling
	* the PageMgr what to do next.  This method handles the details
	* of a command for a page, while the commandAction method in the
	* PageMgr handles the general responsibility of switching displays.
	* @param c a Command object identifying the command.
	* @param d the Displayable on which this event has occurred
	* @return a pageAction code, selected from the public fields defined
	* by PageMgr.
	*/
	public int doCommandAction(Command c,Displayable d) {
		int pageAction;
		UserAccount uAcct;
		if (d != template) {
			throw new IllegalArgumentException(
			"Unknown Displayable received in "+
			this.getClass().getName());
		}
		/*
		* Note that this section of code modifies the UserAccount object.
		* It expects that a whole new list of AccountPages will be built
		* by AccountIndexPage after the new record is written to RMS.
		* The recreated pages will include the "new account" page.  If
		* "new account" is not recreated after this operation, then the
		* next time the page is shown, it will contain the previous
		* "new account" info.
		*/
		if (c == PageMgr.CANCEL_COMMAND) {
			pageAction = PageMgr.CANCEL;
		} else if (c == PageMgr.OK_COMMAND) {
			uAcct = (UserAccount)sObj.getUserData();
			uAcct.setLabel(((TextField)template.get(idxLabel)).getString());
			uAcct.setHost(((TextField)template.get(idxHost)).getString());
			uAcct.setUsername(((TextField)template.get(idxUser)).getString());
			uAcct.setPassword(((TextField)template.get(idxPass)).getString());
			try {
				sObj.writeUserDataObject();
			}
			catch (RecordStoreException e) {
				e.printStackTrace();				// this should be an alert ...
			}
			catch (IOException e) {
				e.printStackTrace();				// this should be an alert ...
			}
			pageAction = PageMgr.ROOT;
		} else {
			throw new IllegalArgumentException(
			"Unknown Command received in "+
			this.getClass().getName());
		}
		return pageAction;
	}
	/**
	* Return the Displayable object for this AccountPage, completely
	* filled in and ready to display.
	* @return the appropriate Displayable object for this Page
	*/
	public Displayable getDisplayable() {
		UserAccount ua = (UserAccount)sObj.getUserData();
		((TextField)template.get(idxLabel)).setString(ua.getLabel());
		((TextField)template.get(idxHost)).setString(ua.getHost());
		((TextField)template.get(idxUser)).setString(ua.getUsername());
		((TextField)template.get(idxPass)).setString(ua.getPassword());
		String t = (getTitle()==null) ? getLabel() : getTitle();
		template.setTitle(t);
		return template;
	}
	//--------------------------------------------------------------------
	
	/** the Displayable template used by this Page */
	static private AccountPageScreen template = new AccountPageScreen();
	/** index within the form for the Label item */
	static private int idxLabel;
	/** index within the form for the Host item */
	static private int idxHost;
	/** index within the form for the Username item */
	static private int idxUser;
	/** index within the form for the Password item */
	static private int idxPass;

	/** the StorableObject that describes the UserAccount record in RMS */
	private StorableObject sObj;

	//----------------------------------------------------------------------
	/**
	* An AccountPageScreen is the Displayable (the view) for an 
	* AccountPage (the model).
	* Note that there is only one AccountPageScreen object actually created,
	* and it gets updated every time an AccountPage is to be displayed.
	*/
	static private class AccountPageScreen extends Form {
		AccountPageScreen() {
			super("Account Template");
			idxLabel = append(new TextField("Label","",100,TextField.ANY));
			idxHost = append(new TextField("Host","",100,TextField.ANY));
			idxUser = append(new TextField("Username","",100,TextField.ANY));
			idxPass = append(new TextField("Password","",100,TextField.PASSWORD));
			addCommand(PageMgr.CANCEL_COMMAND);
			addCommand(PageMgr.OK_COMMAND);
		}
	}
}
